@@ -1,6 +1,6 @@
 #ifndef DICOMINFO_H_INCLUDED
 #define DICOMINFO_H_INCLUDED
-// 	$Id: dicomInfo.h,v 1.14 2007/02/20 20:18:07 valerio Exp $
+// 	$Id: dicomInfo.h,v 1.23 2007/11/15 16:20:44 valerio Exp $
 
 //============================================================================
 // CTN software is curtesy of
@@ -23,27 +23,16 @@
 //
 //****************************************************************************
 
-#ifdef LINUX
-#define ARCHITECTURE "LITTLE_ENDIAN_ARCHITECTURE"
-#define OS "LINUX"
-#elif OSX
-#define ARCHITECTURE "BIG_ENDIAN_ARCHITECTURE"
-#define OS "OSX"
-#endif
-
-#define SHORTSIZE 16
-#define LONGSIZE 32
-
 #include <list>
 #include <map>
 #include <math.h>
 #include <string>
 #include <vector>
-#include "ctn/dicom.h"
-#include "ctn/lst.h"
-#include "ctn/dicom_objects.h"
-#include "ctn/utility.h"
-#include "ctn/condition.h"
+#include "dicom.h"
+#include "lst.h"
+#include "dicom_objects.h"
+#include "utility.h"
+#include "condition.h"
 
 typedef std::string::size_type size_type;
 
@@ -57,11 +46,12 @@ public:
 	void AdjustSet(int numSlices, int imgDiff)
     {
 		numSlices_ = numSlices;
-		imageNum_ -= numSlices * (ACQNum_ - 1) + imgDiff;
+		imageNum_ -= numSlices * (acqNum_ - 1) + imgDiff;
 	}
-	void AdjustSet(bool sagDir, bool corDir, bool traDir, int numSlices, int imgDiff);
-	
+
 	std::string &ImagePath()        { return imgPath_; }
+	std::string &SubjectID()        { return subjID_; }
+	std::string &SubjectName()      { return subjName_; }
 	int ImageNumber() const			{ return imageNum_; }
 	bool Mosaic() const				{ return mosaic_; }
 	bool D3() const                 { return d3_; }
@@ -72,16 +62,20 @@ public:
 	std::string &Comments()		 	{ return comments_; }
 	int Columns() const				{ return columns_; }
 	int Rows() const			 	{ return rows_; }
-	int NumChannels() const		 	{ return numChannels_; }
 	float XFOV() const		 	    { return xFOV_; }
 	float YFOV() const				{ return yFOV_; }
-	float Time() const		 		{ return time_; }
 	int RepetitionTime() const		{ return repTime_; }
 	std::string &SeriesID()    		{ return seriesID_; }
 	int FrequencyDim() const        { return (phaseEncoding_ == 2 ? 1 : 2); }
 	int PhaseDim() const            { return phaseEncoding_; }
 	std::string &SeriesDescription(){ return seriesDescription_; }
-	int ACQNumber() const	        { return ACQNum_; }
+	std::string &SeriesNumber()     { return seriesNumber_; }
+	std::string &SeriesDate()       { return seriesDate_; }
+	std::string &SeriesTime()       { return seriesTime_; }
+	std::string &ACQDescription()   { return acqDescription_; }
+	std::string &Accession()        { return accession_; }
+	int ACQNumber() const	        { return acqNum_; }
+	int ACQEchoNumber() const	    { return acqEchoNum_; }
 	float RowSagCos() const    	 	{ return rowCosines_.at(0); }
 	float RowCorCos() const        	{ return rowCosines_.at(1); }
 	float RowTraCos() const		 	{ return rowCosines_.at(2); }
@@ -112,6 +106,7 @@ private:
 	
 	bool GetImageNumber();
 	bool GetACQNumber();
+	bool GetACQEchoNumber();
 	bool GetMosaic();
 	bool GetD3();
 	bool GetNumSlices();
@@ -123,7 +118,6 @@ private:
 	bool GetPosNormSiemens(char *storage);
 	bool GetColumns();
 	bool GetRows();
-	bool GetNumChannels();
 	bool GetFOV();
 	bool GetTime();
 	bool GetRepetitionTime();
@@ -132,7 +126,14 @@ private:
 	bool GetPhaseEncoding();
 	bool GetManufacturer();
 	bool GetSeriesDescription();
+	bool GetACQDescription();
 	bool MosaicAdjustments();
+	bool GetSubjectID();
+	bool GetSubjectName();
+	bool GetSeriesDate();
+	bool GetSeriesTime();
+	bool GetAccession();
+	bool GetSeriesNumber();
 	
 	U32 GetData(DCM_TAG tag, char **storage);
 	bool GetData(DCM_TAG tag, U32 *storage);
@@ -141,6 +142,8 @@ private:
   
 	DCM_OBJECT *handle_;
 	std::string imgPath_;
+	std::string subjID_;
+	std::string subjName_;
 	int imageNum_;
 	int numGroups_;
 	bool mosaic_;
@@ -148,19 +151,24 @@ private:
 	float sliceThickness_;
 	float sliceSpacing_;
 	std::string comments_;
-	int ACQNum_;
+	int acqNum_;
+	int acqEchoNum_;
 	int columns_;
 	int rows_;
-	int numChannels_;
 	float xFOV_;
 	float yFOV_;
-	float time_;
 	int repTime_;
 	int phaseEncoding_;
 	unsigned short *volumeData_;
 	std::string seriesID_;
 	std::string seriesDescription_;
+	std::string seriesNumber_;
+	std::string acqDescription_;
+	std::string seriesDate_;
+	std::string seriesTime_;
 
+	std::string accession_;
+	
 	enum MANUFACTURER {SIEMENS, GE, UNDEFINED};
 	MANUFACTURER manufacturer_;
 	
@@ -192,13 +200,17 @@ typedef std::map<int, IMAGELIST *> IMAGEMAP;
 
 struct SeriesInfo
 {
-	SeriesInfo() : mosaic(false), maxImgNum(0), minImgNum(0), maxAcqNum(0), numSlices(0) {}
+	SeriesInfo() : mosaic(false), multiEcho(false),
+				   maxImgNum(0), minImgNum(0),
+				   maxAcqNum(0), minAcqNum(0), numSlices(0) {}
 
 	std::string outName;
 	bool mosaic;
+	bool multiEcho;
 	int maxImgNum;
 	int minImgNum;
 	int maxAcqNum;
+	int minAcqNum;
 	int numSlices;
 	IMAGEMAP imageMap;
 };
