@@ -1,6 +1,6 @@
 #ifndef DINIFTI_H_INCLUDED
 #define DINIFTI_H_INCLUDED
-//  $Id: dinifti.h,v 1.10 2007/04/09 17:38:56 valerio Exp $	
+//  $Id: dinifti.h,v 1.11 2007/10/15 17:16:28 valerio Exp $	
 
 //============================================================================
 // CTN software is curtesy of
@@ -43,6 +43,7 @@ class DiniftiOptions
 public:
 	DiniftiOptions()
 		:useSerDesc(false),
+		 nameFormat(NULL),
 		 verbose(false),
 		 noact(false),
 		 numSlices(0),
@@ -50,6 +51,7 @@ public:
 		 niftiFormat(NIfTI_SINGLE)
 	{}
 	bool useSerDesc;
+	char *nameFormat;
 	bool verbose;
 	bool noact;
 	int numSlices;
