@@ -1,4 +1,4 @@
-// 	$Id: dicomInfo.cc,v 1.25 2007/04/09 18:01:02 valerio Exp $
+// 	$Id: dicomInfo.cc,v 1.33 2007/10/15 17:15:46 valerio Exp $
 
 //============================================================================
 // DCM is part of the CTN library, curtesy of
@@ -68,21 +68,20 @@ DICOMImage::DICOMImage(const char *path) : error_(false),
 										   numGroups_(1),
 										   mosaic_(false),
 										   d3_(false),
-										   yFOV_(-1),
-										   xFOV_(-1),
 										   sliceThickness_(0.0),
 										   sliceSpacing_(0.0),
-										   ACQNum_(0),
-										   numSlices_(0),
+										   acqNum_(0),
+										   acqEchoNum_(0),
 										   columns_(-1),
 										   rows_(-1),
-										   numChannels_(1),
-										   time_(0.0),
+										   xFOV_(-1),
+										   yFOV_(-1),
 										   repTime_(0),
 										   phaseEncoding_(0),
 										   volumeData_(NULL),
+										   manufacturer_(UNDEFINED),
 										   shadowSet_(NULL),
-										   manufacturer_(UNDEFINED)
+										   numSlices_(0)
 {
 	bool retValues = true;
 
@@ -90,7 +89,7 @@ DICOMImage::DICOMImage(const char *path) : error_(false),
 	{
 		error_ = true;
 		string msg = string("DICOMImage::DICOMImage: Cannot open file: ") + string(path);
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 	}
 	else
 	{
@@ -102,6 +101,7 @@ DICOMImage::DICOMImage(const char *path) : error_(false),
 			&& GetManufacturer()
 			&& GetImageNumber()
 			&& GetACQNumber()
+			&& GetACQEchoNumber()
 			&& GetD3()
 			&& GetSliceThickness()
 			&& GetNumSlices()
@@ -113,13 +113,18 @@ DICOMImage::DICOMImage(const char *path) : error_(false),
 			&& GetPosNorm()
 			&& GetSliceSpacing()
 			&& GetComments()
-			&& GetNumChannels()
-			&& GetTime()
 			&& GetRepetitionTime()
 			&& GetPhaseEncoding()
 			&& GetNumGroups()
 			&& GetSeriesDescription()
-			&& GetSeriesID();
+			&& GetSeriesNumber()
+			&& GetSeriesID()
+			&& GetACQDescription()
+			&& GetSubjectID()
+			&& GetSubjectName()
+			&& GetSeriesDate()
+			&& GetSeriesTime()
+			&& GetAccession();
 	}
 
 	if (handle_ != NULL)
@@ -191,44 +196,6 @@ bool DICOMImage::OpenFile(const char *path)
 	return retValue;
 }
 
-//****************************************************************************
-//
-// Purpose: Adjust values that can be computed only after the full set has been
-//          acquired.
-//   
-// Parameters: bool sagDir   - does set increment in sagital direction ?
-//             bool corDir   - does set increment in coronal direction ?
-//             bool traDir   - does set increment in transvers direction ?
-//             int numSlices - how many slices per time point
-//             int imgDiff   - number used to adjust image number
-//   
-// Returns: 
-//   
-// Notes: 
-//
-//****************************************************************************
-
-void DICOMImage::AdjustSet(bool sagDir, bool corDir, bool traDir, int numSlices, int imgDiff)
-{
-	if (sagDir)
-		sagNorm_ *= (sagNorm_ < 0 ? -1 : 1);
-	else
-		sagNorm_ *= (sagNorm_ < 0 ? 1 : -1);
-
-	if (corDir)
-		corNorm_ *= (corNorm_ < 0 ? -1 : 1);
-	else
-		corNorm_ *= (corNorm_ < 0 ? 1 : -1);
-
-	if (traDir)
-		traNorm_ *= (traNorm_ < 0 ? -1 : 1);
-	else
-		traNorm_ *= (traNorm_ < 0 ? 1 : -1);
-
-	numSlices_ = numSlices;
-	imageNum_ -= numSlices * (ACQNum_ - 1) + imgDiff;
-}
-
 //////////////////////////////////////////////////////////////////////////////
 ///
 /// Purpose: Get the name of the manufacturer (0008,0070)
@@ -248,7 +215,7 @@ bool DICOMImage::GetManufacturer()
    {
 	  string msg("DICOMImage::GetManufacturer: Cannot get value of manufacturer in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (data != NULL) delete [] data;
 	  return false;
@@ -263,51 +230,6 @@ bool DICOMImage::GetManufacturer()
    return true;
 }
 
-//////////////////////////////////////////////////////////////////////////////
-///
-/// Purpose: Get the value for the image time name (0008,0033)
-///   
-/// @param   NONE
-///   
-/// @return  false if error
-///   
-/// Notes: DICOM stores the time in a string formated as hhmmss.fraction
-//         we will store it in double representing the seconds.
-///
-//////////////////////////////////////////////////////////////////////////////
-
-bool DICOMImage::GetTime()
-{
-   char *data = NULL;
-   if (GetData(DCM_IDIMAGETIME, &data) == 0)
-   {
-	  string msg("DICOMImage::GetTime: Cannot get value of time in image ");
-	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
-	  error_ = true;
-	  if (data != NULL) delete [] data;
-	  return false;
-   }
-
-   // Break it up and parse it
-
-   float strTime = atof(data);
-   time_ = strTime - floor(strTime);
-   strTime -= time_;
-   int secs = (int(strTime)) % 100;
-   time_ += (float)secs;
-   strTime -= (float)secs;
-   strTime /= 100;
-   int mins = (int(strTime)) % 100;
-   time_ += (float)mins * 60;
-   strTime -= (float)mins;
-   strTime /= 100;
-   time_ += strTime * 3600;
-
-   delete [] data;
-   return true;
-}
-
 //////////////////////////////////////////////////////////////////////////////
 ///
 /// Purpose: Get the value for the acquisition repetiton time (0018,0080)
@@ -331,7 +253,7 @@ bool DICOMImage::GetRepetitionTime()
    {
 	  string msg("DICOMImage::GetRepetitonTime: Cannot get value of acquisition repetition time in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (data != NULL) delete [] data;
 	  return false;
@@ -365,7 +287,7 @@ bool DICOMImage::GetPhaseEncoding()
    {
 	  string msg("DICOMImage::GetPhaseEncoding: Cannot get value of phase encoding direction time in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (data != NULL) delete [] data;
 	  return false;
@@ -400,7 +322,7 @@ bool DICOMImage::GetSeriesID()
    {
 	  string msg("DICOMImage::GetSeriesID: Cannot get value of series id in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -413,31 +335,348 @@ bool DICOMImage::GetSeriesID()
 
 //////////////////////////////////////////////////////////////////////////////
 ///
-/// Purpose: Get the value for the series description (0008, 103e)
+/// Purpose: Build an acquisition description string like SPM5
 ///   
 /// @param   NONE 
 ///   
 /// @return  false if error
 ///   
+/// Notes: SPM5 concatenates a number of different parameters.
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetACQDescription()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_ACQTRANSMITTINGCOIL, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ = strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+   
+   if (GetData(DCM_ACQMRACQUISITIONTYPE, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ += string(" ") + strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if (GetData(DCM_ACQSCANNINGSEQUENCE, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ += string(" ") + strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if (GetData(DCM_ACQREPETITIONTIME, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ += string(" TR=") + strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if (GetData(DCM_ACQECHOTIME, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ += string("/TE=") + strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if (GetData(DCM_ACQFLIPANGLE, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+	   acqDescription_ += string("/FA=") + strn;
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   tm acqDatTime;
+   memset(&acqDatTime, 0, sizeof(acqDatTime));
+   
+   if (GetData(DCM_IDACQUISITIONDATE, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+   {
+	   acqDatTime.tm_mday = atoi(&(strn[6]));
+	   strn[6] = '\0';
+	   acqDatTime.tm_mon = atoi(&(strn[4])) - 1;
+	   strn[4] = '\0';
+	   acqDatTime.tm_year = atoi(strn) - 1900;
+   }
+
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if (GetData(DCM_IDACQUISITIONTIME, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+   }
+   else
+   {
+	   // In theory this could be just the time or the date + time
+	   
+	   char *dec = strchr(strn, '.');
+	   if ( dec != NULL )
+	   {
+		   *dec = '\0';
+		   dec -= 2;
+		   acqDatTime.tm_sec = atoi(dec);
+		   *dec = '\0';
+		   dec -= 2;
+		   acqDatTime.tm_min = atoi(dec);
+		   *dec = '\0';
+		   dec -= 2;
+		   acqDatTime.tm_hour = atoi(dec);
+	   }
+   }
+   
+   if ( strn != NULL )
+   {
+	   delete [] strn;
+	   strn = NULL;
+   }
+
+   if ( (acqDatTime.tm_mday != 0) || (acqDatTime.tm_mon != 0) || (acqDatTime.tm_year != 0)
+		|| (acqDatTime.tm_sec != 0) || (acqDatTime.tm_min != 0) || (acqDatTime.tm_hour != 0) )
+   {
+	   strn = new char[32];
+	   strftime(strn, 31, "%d-%b-%Y %H:%M:%S", &acqDatTime);
+	   acqDescription_ += string(" ") + strn;
+	   delete [] strn;
+	   strn = NULL;
+   }
+   
+   if ( mosaic_ ) acqDescription_ += string(" Mosaic");
+
+   if ( acqDescription_.length() == 0 )
+	   acqDescription_ = "DINIfTI";
+
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the subject ID (0010, 0020)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
 /// Notes: 
 ///
 //////////////////////////////////////////////////////////////////////////////
 
+bool DICOMImage::GetSubjectID()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_PATID, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  subjID_ = "";
+   }
+   else
+	   subjID_ = strn;
+   delete [] strn;
+   DeSpace(subjID_);
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the subject name (0010, 0010)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
+/// Notes: 
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetSubjectName()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_PATNAME, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  subjName_ = "";
+   }
+   else
+	   subjName_ = strn;
+   delete [] strn;
+   DeSpace(subjName_);
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the series date (0008, 0021)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
+/// Notes: 
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetSeriesDate()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_IDSERIESDATE, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  seriesDate_ = "";
+   }
+   else
+	   seriesDate_ = strn;
+   delete [] strn;
+   DeSpace(seriesDate_);
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the series time (0008, 0031)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
+/// Notes: Cut off past the minutes
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetSeriesTime()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_IDSERIESTIME, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  seriesTime_ = "";
+   }
+   else
+   {
+	   strn[4] = '\0';
+	   seriesTime_ = strn;
+   }
+   
+   delete [] strn;
+   DeSpace(seriesTime_);
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value accession number (0008, 0050)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
+/// Notes: 
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetAccession()
+{
+   char *strn = NULL;
+  
+   if (GetData(DCM_IDACCESSIONNUMBER, &strn) == 0)
+   {
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  accession_ = "";
+   }
+   else
+	   accession_ = strn;
+   delete [] strn;
+   DeSpace(accession_);
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the series description (0008, 103e)
+///   
+/// @param   NONE 
+///   
+/// @return  always true
+///   
+/// Notes: If we can't find the description, make something up
+///
+//////////////////////////////////////////////////////////////////////////////
+
 bool DICOMImage::GetSeriesDescription()
 {
    char *strn = NULL;
   
    if (GetData(DCM_IDSERIESDESCR, &strn) == 0)
    {
-	  string msg("DICOMImage::GetSeriesDescription: Cannot get value of series id in image ");
-	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
-	  error_ = true;
-	  if (strn != NULL) delete [] strn;
-	  return false;
+	  error_ = false;
+	  COND_PopCondition(TRUE);				// Clear error from stack
+	  seriesDescription_ = "";
    }
-
-   seriesDescription_ = strn;
+   else
+	   seriesDescription_ = strn;
    delete [] strn;
    DeSpace(seriesDescription_);
    return true;
@@ -465,7 +704,7 @@ bool DICOMImage::GetImageNumber()
    {
 	  string msg("DICOMImage::ImageNumber: Cannot get value of image number in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -478,6 +717,41 @@ bool DICOMImage::GetImageNumber()
    return true;
 }
 
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the series number (0020, 0011)
+///   
+/// @param   NONE 
+///   
+/// @return  false if error
+///   
+/// Notes: 
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetSeriesNumber()
+{
+   // The number is really stored as a string
+  
+   char *strn = NULL;
+  
+   if (GetData(DCM_RELSERIESNUMBER, &strn) == 0)
+   {
+	  string msg("DICOMImage::SeriesNumber: Cannot get value of image number in image ");
+	  msg += imgPath_;
+	  cerr << endl << "\t**** " << msg << endl << endl;
+	  error_ = true;
+	  if (strn != NULL) delete [] strn;
+	  return false;
+   }
+
+   // Convert string to numerical value
+
+   seriesNumber_ = strn;
+   delete [] strn;
+   return true;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 ///
 /// Purpose: Get the value for the relative acquisition number (0020, 0012)
@@ -500,7 +774,7 @@ bool DICOMImage::GetACQNumber()
    {
 	  string msg("DICOMImage::GetACQNumber: Cannot get value of image number in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -508,7 +782,42 @@ bool DICOMImage::GetACQNumber()
 
    // Convert string to numerical value
 
-   ACQNum_ = atoi(strn);
+   acqNum_ = atoi(strn);
+   delete [] strn;
+   return true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+///
+/// Purpose: Get the value for the acquisition echo number (0018, 0086)
+///   
+/// @param   NONE 
+///   
+/// @return  false if error
+///   
+/// Notes: This applies to field maps
+///
+//////////////////////////////////////////////////////////////////////////////
+
+bool DICOMImage::GetACQEchoNumber()
+{
+   // The number is really stored as a string
+  
+   char *strn = NULL;
+  
+   if (GetData(DCM_ACQECHONUMBER, &strn) == 0)
+   {
+	  string msg("DICOMImage::GetACQEchoNumber: Cannot get value of echo number in image ");
+	  msg += imgPath_;
+	  cerr << endl << "\t**** " << msg << endl << endl;
+	  error_ = true;
+	  if (strn != NULL) delete [] strn;
+	  return false;
+   }
+
+   // Convert string to numerical value
+
+   acqEchoNum_ = atoi(strn);
    delete [] strn;
    return true;
 }
@@ -535,7 +844,7 @@ bool DICOMImage::GetImgOrientation()
 	{
 		string msg("DICOMImage::GetImgOrientation: Cannot get value of patient orientation in image ");
 		msg += imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 		error_ = true;
 		if (strn != NULL) delete [] strn;
 		return false;
@@ -545,7 +854,6 @@ bool DICOMImage::GetImgOrientation()
 	// Two sets of direction cosines (x, y, z) for first row and first column
 	// with respect to the patient, separated by back slashes.
 	// First set is row values, second the colum values
-	// The z values are inverse sign ?
 
 	char *tok = strtok(strn, "\\");
 	rowCosines_.push_back(atof(tok));
@@ -587,7 +895,7 @@ bool DICOMImage::GetMosaic()
    {
 	  string msg("DICOMImage::Mosaic: Cannot get value of image type in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -622,7 +930,7 @@ bool DICOMImage::GetD3()
    {
 	  string msg("DICOMImage::GetD3: Cannot get value of acquisition type in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -657,7 +965,7 @@ bool DICOMImage::GetSliceThickness()
    {
 	  string msg("DICOMImage::SliceThickness: Cannot get value of slice thickness in image ");
 	  msg += imgPath_;
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
+	  cerr << endl << "\t**** " << msg << endl << endl;
 	  error_ = true;
 	  if (strn != NULL) delete [] strn;
 	  return false;
@@ -954,7 +1262,7 @@ bool DICOMImage::GetNumGroups()
 	{
 		string msg("DICOMImage::NumGroups: Cannot get value of Siemens info in image ");
 		msg += imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 		error_ = true;
 		delete [] storage;
 		storage = NULL;
@@ -967,63 +1275,6 @@ bool DICOMImage::GetNumGroups()
 	return true;
 }
 
-//////////////////////////////////////////////////////////////////////////////
-///
-/// Purpose: Get the number of channels from shadow set
-///   
-/// @param   NONE
-///   
-/// @return  false if error
-///   
-/// Notes: 
-///
-//////////////////////////////////////////////////////////////////////////////
-
-bool DICOMImage::GetNumChannels()
-{
-	char *storage = GetShadowSet();
-	if ((!mosaic_) || (manufacturer_ != SIEMENS) || (storage == NULL) )
-	{
-		numChannels_ = 1;
-		return true;
-	}
-	
-	bool found = false;
-
-	// We need to find the greatest value assigned to channels
-	  
-	char *tok = strtok(storage, "\n");
-	while (tok)
-	{
-		if (! strncmp(tok, "sCOIL_SELECT_MEAS.asList[", 25))
-		{
-			char *valuep = strstr(tok, "lRxChannelConnected");
-			if (valuep != NULL)
-			{
-				valuep = strstr(tok, "=");
-				sscanf(valuep, "=%i", &numChannels_);
-				found = true;
-			}
-		}
-		else if (found)					// If we've found it, why bother to continue.
-			break;
-		
-		tok = strtok(NULL, "\n");
-	}
-
-	if (! found)
-	{
-		string msg("DICOMImage::GetNumChannels: Cannot get value of Siemens info in image ");
-		msg += imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
-		error_ = true;
-	}
-
-	delete [] storage;
-	storage = NULL;
-	return found;
-}
-
 //////////////////////////////////////////////////////////////////////////////
 ///
 /// Purpose: If this is a Siemens image read the shadow set (0029, 1020) and
@@ -1031,7 +1282,7 @@ bool DICOMImage::GetNumChannels()
 ///   
 /// @param   NONE
 ///   
-/// @return  false if error
+/// @return  Always true
 ///   
 /// Notes: If this is not mosaic, we will have to fix it later.
 ///
@@ -1046,64 +1297,76 @@ bool DICOMImage::GetNumSlices()
 		{
 			numSlices_ = 1;
 			error_ = true;
-			return false;
 		}
-  
-		bool found = false;
-	  
-		// Believe it or not, we are looking only for one thing ....
-	  
-		char *tok = strtok(storage, "\n");
-		while (tok)
+		else
 		{
-			if ( mosaic_ || !d3_ )
+			bool found = false;
+	  
+			// Believe it or not, we are looking only for one thing ....
+	  
+			char *tok = strtok(storage, "\n");
+			while (tok)
 			{
-				if (! strncmp(tok, "sSliceArray.lSize", 17))
+				if ( mosaic_ || !d3_ )
 				{
-					char *valuep = strstr(tok, "=");
-					sscanf(valuep, "=%i", &numSlices_);
-					found = true;
-					break;
+					if (! strncmp(tok, "sSliceArray.lSize", 17))
+					{
+						char *valuep = strstr(tok, "=");
+						sscanf(valuep, "=%i", &numSlices_);
+						found = true;
+						break;
+					}
 				}
-			}
-			else
-			{
-				if (! strncmp(tok, "sSliceArray.asSlice[0].dThickness", 33))
+				else
 				{
-					char *valuep = strstr(tok, "=");
-					float fval;
-					sscanf(valuep, "=%f", &fval);
-					numSlices_ = int(0.5 + fval / sliceThickness_);
-					found = true;
-					break;
+					if (! strncmp(tok, "sSliceArray.asSlice[0].dThickness", 33))
+					{
+						char *valuep = strstr(tok, "=");
+						float fval;
+						sscanf(valuep, "=%f", &fval);
+						numSlices_ = int(0.5 + fval / sliceThickness_);
+						found = true;
+						break;
+					}
 				}
-			}
 		
-			tok = strtok(NULL, "\n");
-		}
+				tok = strtok(NULL, "\n");
+			}
 
-		if (! found)
-		{
-			string msg("DICOMImage::NumSlices: Cannot get value of Siemens info in image ");
-			msg += imgPath_;
-			cerr << endl << "\t^G^G**** " << msg << endl << endl;
-			error_ = true;
-		}
+			if (! found)
+				error_ = true;
 
-		delete [] storage;
-		storage = NULL;
-		return found;
+			delete [] storage;
+			storage = NULL;
+		}
 	}
 	else
 	{
 		char *strn = NULL;
 		if (GetData(DCM_RELIMAGESINACQUISITION, &strn) == 0)
+		{
 			numSlices_ = 1;
+			COND_PopCondition(TRUE);				// Clear error from stack
+			error_ = true;
+		}
 		else
+		{
 			numSlices_ = atoi(strn);
+		}
+		
 		delete [] strn;
-		return true;
 	}
+
+	if ( error_ )
+	{
+		error_ = false;
+		string msg("DICOMImage::NumSlices: Cannot get number of slices from ");
+		msg += imgPath_;
+		cerr << endl << "\t**** " << msg << endl
+			 << "\t**** Setting value to 1 (can be changed with '-s' option on command line.)" << endl << endl;
+	}
+
+	return true;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1129,7 +1392,7 @@ bool DICOMImage::GetPosNorm()
 
 	char *storage = NULL;
 	
-	if ( (manufacturer_ == SIEMENS) && ((storage = GetShadowSet()) != NULL) )
+	if ( (mosaic_) && (manufacturer_ == SIEMENS) && ((storage = GetShadowSet()) != NULL) )
 		return GetPosNormSiemens(storage);
 	else
 	{
@@ -1258,8 +1521,10 @@ bool DICOMImage::GetPosNormSiemens(char *storage)
 			return false;
 		}
 
-	float rowProp = (((float)rows_ / 2) - 0.5) * ((float)xFOV_ / rows_);
-	float colProp = (((float)columns_ / 2) - 0.5) * ((float)yFOV_ / columns_);
+	// Siemens draws the images starting at x=-FOV/2, y=-FOV/2
+	// therefore, we need to move FOV/2 from the center of the slice
+	float rowProp = ((float)rows_ / 2)    * ((float)xFOV_ / rows_);
+	float colProp = ((float)columns_ / 2) * ((float)yFOV_ / columns_);
 	
 	float sagAdjust = - rowProp * rowCosines_.at(0) - colProp * colCosines_.at(0);
 	float corAdjust = - rowProp * rowCosines_.at(1) - colProp * colCosines_.at(1);
@@ -1299,50 +1564,50 @@ bool DICOMImage::GetPosNormSiemens(char *storage)
 
 U32 DICOMImage::GetData(DCM_TAG tag, char **storage)
 {
-   DCM_ELEMENT element;
-   U32 len;
+	DCM_ELEMENT element;
+	U32 len;
   
-   if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL
+	if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL
 		|| DCM_GetElementSize(&handle_, tag, &len) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return 0;
-   }
+	{
+		error_ = true;
+		return 0;
+	}
 
-   void *ctx = NULL;
-   if (*storage == NULL)
-	   *storage = new char [len+1];
+	void *ctx = NULL;
+	if (*storage == NULL)
+		*storage = new char [len+1];
    
-   element.d.string = *storage;
+	element.d.string = *storage;
 
-   U32 newLen;
-   if (DCM_GetElementValue(&handle_, &element, &newLen, &ctx) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return 0;
-   }
+	U32 newLen;
+	if (DCM_GetElementValue(&handle_, &element, &newLen, &ctx) != DCM_NORMAL)
+	{
+		error_ = true;
+		return 0;
+	}
 
-   if (newLen > len)
-   {
-	  string msg("DICOMImage::GetData: Inconsistent data size");
-	  cerr << endl << "\t^G^G**** " << msg << endl << endl;
-	  error_ = true;
-	  return 0;
-   }
+	if (newLen > len)
+	{
+		string msg("DICOMImage::GetData: Inconsistent data size");
+		cerr << endl << "\t**** " << msg << endl << endl;
+		error_ = true;
+		return 0;
+	}
 
-   // Terminate and strip trailing blanks
+	// Terminate and strip trailing blanks
 
-   (*storage)[len] = '\0';
+	(*storage)[len] = '\0';
 
-   if (len > 0)
-   {
-	  while ((len > 0) && ((*storage)[--len] == ' '))
-		 (*storage)[len] = '\0';
+	if (len > 0)
+	{
+		while ((len > 0) && ((*storage)[--len] == ' '))
+			(*storage)[len] = '\0';
   
-	  return len+1;
-   }
-   else
-	  return 0;
+		return len+1;
+	}
+	else
+		return 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1360,23 +1625,23 @@ U32 DICOMImage::GetData(DCM_TAG tag, char **storage)
 
 bool DICOMImage::GetData(DCM_TAG tag, U32 *storage)
 {
-   DCM_ELEMENT element;
-   if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return false;
-   }
+	DCM_ELEMENT element;
+	if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL)
+	{
+		error_ = true;
+		return false;
+	}
 
-   U32 len;
-   void *ctx = NULL;
-   element.d.ul = storage;
-   if (DCM_GetElementValue(&handle_, &element, &len, &ctx) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return false;
-   }
+	U32 len;
+	void *ctx = NULL;
+	element.d.ul = storage;
+	if (DCM_GetElementValue(&handle_, &element, &len, &ctx) != DCM_NORMAL)
+	{
+		error_ = true;
+		return false;
+	}
   
-   return true;
+	return true;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1394,23 +1659,23 @@ bool DICOMImage::GetData(DCM_TAG tag, U32 *storage)
 
 bool DICOMImage::GetData(DCM_TAG tag, U16 *storage)
 {
-   DCM_ELEMENT element;
-   if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return false;
-   }
+	DCM_ELEMENT element;
+	if (DCM_GetElement(&handle_, tag, &element) != DCM_NORMAL)
+	{
+		error_ = true;
+		return false;
+	}
 
-   U32 len;
-   void *ctx = NULL;
-   element.d.us = storage;
-   if (DCM_GetElementValue(&handle_, &element, &len, &ctx) != DCM_NORMAL)
-   {
-	  error_ = true;
-	  return false;
-   }
+	U32 len;
+	void *ctx = NULL;
+	element.d.us = storage;
+	if (DCM_GetElementValue(&handle_, &element, &len, &ctx) != DCM_NORMAL)
+	{
+		error_ = true;
+		return false;
+	}
   
-   return true;
+	return true;
 }
 
 //****************************************************************************
@@ -1438,7 +1703,7 @@ bool DICOMImage::ReadSlice(U16 *data, int slice)
 	{
 		error_ = true;
 		string msg = string("DICOMImage::ReadSlice: Trying to read past end of file: ") + imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 		return false;
 	}
 
@@ -1493,7 +1758,7 @@ bool DICOMImage::ReadVolume()
 		{
 			error_ = true;
 			string msg = string("DICOMImage::ReadVolume: Cannot open file: ") + imgPath_;
-			cerr << endl << "\t^G^G**** " << msg << endl << endl;
+			cerr << endl << "\t**** " << msg << endl << endl;
 			return false;
 		}
 	}		
@@ -1503,7 +1768,7 @@ bool DICOMImage::ReadVolume()
 	{
 		error_ = true;
 		string msg = string("DICOMImage::ReadVolumes: Error reading file: ") + imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 		return false;
 	}
 
@@ -1519,7 +1784,7 @@ bool DICOMImage::ReadVolume()
 	{
 		error_ = true;
 		string msg = string("DICOMImage::ReadVolume: Error reading file: ") + imgPath_;
-		cerr << endl << "\t^G^G**** " << msg << endl << endl;
+		cerr << endl << "\t**** " << msg << endl << endl;
 
 		delete [] volumeData_;
 		volumeData_ = NULL;
