@@ -1,14 +1,14 @@
-//  $Id: dinifti.cc,v 1.31 2007/05/22 14:39:25 valerio Exp $
+//  $Id: dinifti.cc,v 1.50 2007/11/26 21:07:05 valerio Exp $
 
-#define VERSION "2.27"
+#define VERSION "2.28.10"
 
 //****************************************************************************
 //
 // Modification History (most recent first)
 // mm/dd/yy  Who  What
 //
-// 10/04/06  Add DiniftiOptions class and verbose mode
-//           (contributed by Michael Hanke)
+// 10/04/06       Add DiniftiOptions class and verbose mode
+//                (contributed by Michael Hanke)
 // 08/15/06  VPL  Fix bugs in output
 //                Make software GPL
 //                Format --help and --version to be in standard form
@@ -24,8 +24,6 @@
 #include "dinifti.h"
 #include "niftiout.h"
 
-
-
 //****************************************************************************
 //
 // Purpose: Comaprison of image entries
@@ -35,7 +33,7 @@
 //   
 // Returns: 0 if rhs >= lhs
 //   
-// Notes: Adapted from n2n2.c written by L&R Fleisher
+// Notes: 
 //
 //****************************************************************************
 
@@ -77,6 +75,7 @@ static int CompareImages(const DICOMImage &lhs, const DICOMImage &rhs)
 static void SetFileName(char *&fname, const char *name, OUTPUT_TYPE form, bool compressed, bool header = true)
 {
 	ostringstream outFileName(name, ostringstream::app);
+	
 	if (header)
 		switch( form )
 		{
@@ -101,6 +100,76 @@ static void SetFileName(char *&fname, const char *name, OUTPUT_TYPE form, bool c
 	strcpy(fname, outFileName.str().c_str());
 }
 
+//****************************************************************************
+//
+// Purpose: Generate output name based on format string
+//   
+// Parameters: DICOMImage *dcmImg - current DICOM image
+//             char *format       - format string
+//             std::string &name  - where to append the format
+//   
+// Returns: NONE
+//   
+// Notes: 
+//
+//****************************************************************************
+
+static void FormattedName(DICOMImage *dcmImg, char *format, std::string &name)
+{
+	// Loop through characters in format and process accordingly
+	char *cp = format;
+	while ( *cp != '\0' )
+	{
+		if ( *cp == '%' )
+		{
+			++cp;
+			switch ( *cp )
+			{
+				case 'I':
+					name += dcmImg->SubjectID();
+					break;
+				case 'N':
+					name += dcmImg->SubjectName();
+					break;
+				case 'S':
+					name += dcmImg->SeriesNumber();
+					break;
+				case 'D':
+					name += dcmImg->SeriesDescription();
+					break;
+				case 'Y':
+					name += dcmImg->SeriesDate();
+					break;
+				case 'T':
+					name += dcmImg->SeriesTime();
+					break;
+				case 'A':
+					name += dcmImg->Accession();
+					break;
+				case 'C':
+					name += dcmImg->Comments();
+					break;
+				case '%':
+					name += '%';
+					break;
+				default:
+					// Who knows, ignore it
+					break;
+			}
+		}
+		else
+		{
+			// Just append the character to the string
+			// But use '~' instead of '~'
+			if ( *cp == '/' )
+				name += '~';
+			else
+				name += *cp;
+		}
+		++cp;
+	}
+}
+
 //****************************************************************************
 //
 // Purpose: Display usage and leave
@@ -123,6 +192,24 @@ void ShowHelp(char **argv)
 		 << "                      'n2' - NIfTI-2 Dual file" << endl 
 		 << "                      'n1' - NIfTI-1 Single file **Default**" << endl 
 		 << " -d                   append series description to output file name(s)" << endl
+		 << " --name=\"<format>\"  format output file name(s) according to <format>" << endl
+		 << "                      where <format> : " << endl
+		 << "                                       %I = subject ID" << endl
+		 << "                                       %N = subject name" << endl
+		 << "                                       %S = series number" << endl
+		 << "                                       %D = series description" << endl
+		 << "                                       %Y = series date" << endl
+		 << "                                       %T = series time" << endl
+		 << "                                       %A = accession number" << endl
+		 << "                                       %C = image comments" << endl		
+		 << "                                       %% = percentage sign" << endl
+		 << "                      Example: " << endl
+		 << "                               --name=\"%Y:%T - %A - %I - %D(%S)\"" << endl
+		 << "                      will result in name(s) formed like" << endl
+		 << "                               date:time - accession - ID - description(number)" << endl
+ 		 << "                      N.B.: format string must be enclosed in quotes" << endl
+		 << "                            if format is specified, -d flag is ignored" << endl
+		 << "                            the reserved character \"/\" will be rendered as \"~\"" << endl
 		 << " -s #                 number of slices per volume" << endl
 		 << " -v --verbose    enable verbose status output" << endl
 		 << " -n --noact      do all processing, but do not write files" << endl << endl
@@ -196,6 +283,11 @@ int main(int argc, char **argv)
 		{
 			opts.useSerDesc = true;
 		}
+		else if (! strncmp(argv[parseArg], "--name=", 7) )
+		{
+			opts.nameFormat = new char [strlen(argv[parseArg]) - 6];
+			strcpy(opts.nameFormat, &(argv[parseArg][7]));
+		}
 		else if ( (strcmp(argv[parseArg], "-v") == 0) || (strcmp(argv[parseArg], "--verbose") == 0) )
 		{
 			opts.verbose = true;
@@ -276,6 +368,7 @@ int main(int argc, char **argv)
 //
 //             in DiniftiOptions class:
 //             bool useSerDesc            - append series description to name ?
+//             char nameFormat            - specified special format for name ?
 //             int numSlices              - number of slices per volume
 //   
 // Returns: NONE
@@ -296,6 +389,18 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 	// Figure out if the outName is a directory or the name of a file
 
 	const bool outDirectory = (opendir(outName) != NULL);
+
+	// Make sure outname doesn't already have extension, or we end up with
+	// "double" extension (e.g., ".nii.nii"), which FSL doesn't like
+
+	if ( !outDirectory )
+	{
+		char *extPos = strrchr(outName, '.');
+		if ( extPos != NULL )
+			if ( (strcmp(extPos, ".nii") == 0) || (strcmp(extPos, ".nia") == 0) || (strcmp(extPos, ".hdr") == 0) )
+				*extPos = '\0';
+	}
+	
 	int relSeriesNumber = 0;
 
 	if (opts.verbose)
@@ -318,6 +423,7 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 			int imgNum = dcmImg->ImageNumber();
 
 			// The acquisition number can come directly from the header or
+			// If the echo number is greater than 0 use that (+1)
 			// it can be computed from the header (if numSlices > 1) or
 			// it can be computed from the parameter given by the user
 			
@@ -330,6 +436,10 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 			{
 				acqNum = ((imgNum - 1) / dcmImg->NumSlices()) + 1;
 			}
+			else if ( !dcmImg->Mosaic() && (dcmImg->ACQEchoNumber() > 0) )
+			{
+				acqNum = dcmImg->ACQEchoNumber() + 1;
+			}
 			else
 				acqNum = dcmImg->ACQNumber();
 
@@ -348,7 +458,12 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 				si.outName = outName;
 				if (outDirectory)
 				{
-					if ( opts.useSerDesc )
+					if ( (opts.nameFormat != NULL) && (strlen(opts.nameFormat) > 0) )
+					{
+						si.outName += string("/");
+						FormattedName(dcmImg, opts.nameFormat, si.outName);
+					}
+					else if ( opts.useSerDesc )
 					{
 						si.outName += string("/") + dcmImg->SeriesDescription();
 					}
@@ -372,6 +487,11 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 						si.outName += dcmImg->ImagePath().substr(++begName, --endName);
 					}
 				}
+				else if ( (opts.nameFormat != NULL) && (strlen(opts.nameFormat) > 0) )
+				{
+					si.outName += string("+");
+					FormattedName(dcmImg, opts.nameFormat, si.outName);
+				}
 				else if ( opts.useSerDesc )
 				{
 					si.outName += string("+") + dcmImg->SeriesDescription();
@@ -388,13 +508,16 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 			
 			si.mosaic = dcmImg->Mosaic();
 			si.numSlices = dcmImg->NumSlices();
+			si.multiEcho = (dcmImg->ACQEchoNumber() > 0);
+			
 			if (imgNum > si.maxImgNum)
 				si.maxImgNum = imgNum;
-			if ((imgNum < si.minImgNum) || (si.minImgNum == 0))
+			if ( (imgNum < si.minImgNum) || (si.minImgNum == 0) )
 				si.minImgNum = imgNum;
 			if (acqNum > si.maxAcqNum)
 				si.maxAcqNum = acqNum;
-
+			if ( (acqNum < si.minAcqNum) || (si.minAcqNum == 0) )
+				si.minAcqNum = acqNum;
 			seriesMap[dcmImg->SeriesID()] = si;
 		}
 		else
@@ -443,7 +566,7 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 				dupes = count( series_ids.begin(), series_ids.end(), si.outName );
 			}
 			// issue warning to notify the name change
-			cerr << endl << "\t^G^G**** WARNING: Duplicate image series name '" <<
+			cerr << endl << "\t**** WARNING: Duplicate image series name '" <<
 				old << "'" << endl
 				<< "\t**** New image series name is '" << si.outName << "'." << endl;
 
@@ -454,21 +577,26 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 		// store the current image series name
 		series_ids.push_back(si.outName);
 
-		if ( (opts.numSlices == 0) && (si.numSlices == 1) && (si.maxAcqNum == 1) && (si.maxImgNum > si.minImgNum) )
+		if ( (opts.numSlices == 0) &&
+			 (si.numSlices == 1) &&
+			 (si.maxAcqNum == si.minAcqNum) &&
+			 (si.maxImgNum > si.minImgNum) )
 		{
 			cerr << endl << "\t**** StoreImageFiles: Warning: number of slices per volume = 1" << endl
 				 << "\t**** and number of time points = 1." << endl
 				 << "\t**** If incorrect, run program with \"-s\" option." << endl;
 		}
-		
-		if ( numGroups > 1 )
+
+		const int numAcq = (si.multiEcho ? 1 : (si.maxAcqNum - si.minAcqNum + 1));
+		int numImgGroup = (si.maxImgNum - si.minImgNum + 1) / (numGroups * numAcq);
+
+		if ( (numImgGroup > 0) && (numGroups > 1) )
 		{
 			// We need to create "virtual" series and split the images among them
 
 			string origSeriesID = series->first;
-			const int numImgGroup = (si.maxImgNum - si.minImgNum + 1) / (numGroups * si.maxAcqNum);
 
-			// Hold the names in a map for easy retrieval
+            // Hold the names in a map for easy retrieval
 
 			map<int, string> newSeriesNames;
 			for (int group = 1; group <= numGroups; ++group)
@@ -478,12 +606,13 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 				newSeriesNames[group] = newName.str();
 			}
 			
-			for (int acqNum = 1; acqNum <= si.maxAcqNum; ++acqNum)
+			for (IMAGEMAP::iterator timeIter = fullImageMap.begin(); timeIter != fullImageMap.end(); ++timeIter)
 			{
 				// Grab the full list of images for this time point
 				// And split them up among the groups
-				
-				IMAGELIST *timePoint = fullImageMap[acqNum];
+
+				int acqNum = timeIter->first;
+				IMAGELIST *timePoint = timeIter->second;
 				for (IMAGELIST::iterator img = timePoint->begin(); img != timePoint->end(); ++img)
 				{
 					// We know the acqisition number, figure out which group it
@@ -502,6 +631,7 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 						newSeriesInfo.minImgNum = (group - 1) * numImgGroup + si.minImgNum;
 						newSeriesInfo.maxImgNum = group * numImgGroup + si.minImgNum - 1;
 						newSeriesInfo.maxAcqNum = si.maxAcqNum;
+						newSeriesInfo.minAcqNum = si.minAcqNum;
 					}
 					
 					newTimePoint->push_back(*img);
@@ -530,41 +660,17 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 	{
 		SeriesInfo si = series->second;
 		if (!si.mosaic)
-		{
-			const int numSlices = (si.maxImgNum - si.minImgNum + 1) / si.maxAcqNum;
+		{			
+			const int numAcq = (si.multiEcho ? 1 : (si.maxAcqNum - si.minAcqNum + 1));
+			const int numSlices = (opts.numSlices > 0 ? opts.numSlices :
+								   (si.numSlices > 1 ? si.numSlices : (si.maxImgNum - si.minImgNum + 1) / numAcq));
 			
-			for (int acqNum = 1; acqNum <= si.maxAcqNum; ++acqNum)
+			for (IMAGEMAP::iterator timeIter = si.imageMap.begin(); timeIter != si.imageMap.end(); ++timeIter)
 			{
-				IMAGELIST *timePoint = si.imageMap[acqNum];
+				IMAGELIST *timePoint = timeIter->second;
 				timePoint->sort(CompareImages);
-
-				// Get values from first and second image and compute directions
-				// If only one item, don't bother with direction, just fix image number
-
-				if (timePoint->size() > 1)
-				{
-					IMAGELIST::iterator image = timePoint->begin();
-					float x1 = image->SagPos();
-					float y1 = image->CorPos();
-					float z1 = image->TraPos();
-
-					// Get info from second image in same acquisition
-
-					image++;
-					float x2 = image->SagPos();
-					float y2 = image->CorPos();
-					float z2 = image->TraPos();
-
-					bool sagDir = (x2 >= x1);
-					bool corDir = (y2 >= y1);
-					bool traDir = (z2 >= z1);
-		
-					for (image = timePoint->begin(); image != timePoint->end(); image++)
-						image->AdjustSet(sagDir, corDir, traDir, numSlices, si.minImgNum-1);
-				}
-				else
-					for (IMAGELIST::iterator image = timePoint->begin(); image != timePoint->end(); image++)
-						image->AdjustSet(numSlices, si.minImgNum-1);
+				for (IMAGELIST::iterator image = timePoint->begin(); image != timePoint->end(); image++)
+					image->AdjustSet(numSlices, si.minImgNum-1);
 			}
 		}
 	}
@@ -599,29 +705,8 @@ void ProcessSeries(SeriesInfo &seriesInfo, const DiniftiOptions& opts)
 	nifti_image niftiHdr;
 	if (! NIfTICreateHeader(niftiHdr, seriesInfo.imageMap))
 		exit(1);
-	string description("dinifti v");
-	description += VERSION;
-	switch ( opts.niftiFormat )
-	{
-		case ANALYZE:
-			description += " ANALYZE 7.5 file format";
-			break;
-	    case NIfTI_SINGLE:
-			description += " NIfTI-1 Single file format";
-			break;
-	    case NIfTI_DUAL:
-			description += " NIfTI-1 Dual file format";
-			break;
-	}
-	
-	strcpy(niftiHdr.descrip, description.c_str());
 	niftiHdr.nifti_type = (int)opts.niftiFormat;
 
-	// Just one coil mode
-
-	niftiHdr.nvox = niftiHdr.nvox / niftiHdr.nu;
-	niftiHdr.nu = 1;
-
 	// main processing loop over the repeat counters
 
 	SetFileName(niftiHdr.fname, seriesInfo.outName.c_str(), opts.niftiFormat, opts.compressed);
