@@ -1,4 +1,4 @@
-//  $Id: niftiout.cc,v 1.12 2007/04/25 17:37:54 valerio Exp $	
+//  $Id: niftiout.cc,v 1.17 2007/11/15 16:19:42 valerio Exp $	
 
 //****************************************************************************
 //
@@ -39,7 +39,7 @@ static bool SliceDistance(nifti_image &hdr, IMAGELIST *imageList)
 	
 	if (hdr.nz < 2)                     // Only one slice, so use slice thinkness
 	{
-		cerr << endl << "\t^G^G**** SliceDistance: WARNING: Number of slices is less than two" << endl
+		cerr << endl << "\t**** SliceDistance: WARNING: Number of slices is less than two" << endl
 			<< "\t**** Using slice thinkness as slice distance" << endl;
 		hdr.dz = image->SliceThickness();
     }
@@ -108,9 +108,9 @@ static void FindQuaternion(nifti_image &hdr, IMAGELIST *imageList)
 	// offset = slicePosition(center) - M*(i,j,k of center)
 	// since we are working with the first slice k = 0.
 
-	hdr.qoffset_x  = -image->SagPos();
-	hdr.qoffset_y  = -image->CorPos();
-	hdr.qoffset_z  = image->TraPos();
+	hdr.qoffset_x = -image->SagPos();
+	hdr.qoffset_y = -image->CorPos();
+	hdr.qoffset_z =  image->TraPos();
 
 	nifti_mat44_to_quatern(hdr.qto_xyz, &(hdr.quatern_b), &(hdr.quatern_c), &(hdr.quatern_d),
 						   NULL, NULL, NULL, NULL, NULL, NULL, &(hdr.qfac));
@@ -134,9 +134,6 @@ static void FindQuaternion(nifti_image &hdr, IMAGELIST *imageList)
 //
 //    THE FUNCTION FILLS THE DEFAULT VALUES INTO nifti_image AS FOUND IN 
 //    THE INPUT.HERE IS THE LIST OF WHAT ONE MAY WANT TO CHANGE ELSEWHERE:
-//        - nu (the number of coils in output files) is set to number of 
-//          coils in the input file. Depending on user settings this may
-//          not be true. Correct nu and nvox as a result of this change
 //        - nifti_type file format should be set according to user specifications
 //          notice that only float storage type is supported, see a note below
 //
@@ -177,25 +174,21 @@ bool NIfTICreateHeader(nifti_image &hdr, IMAGEMAP &imageMap)
 	hdr.ny = image->Rows();
 	hdr.nz = image->NumSlices();				// number of slices per volume
 	hdr.nt = imageMap.size(); 					// number of timepoints
-	hdr.nu = image->NumChannels(); 		   		// this will change depending on coil combination method
+	hdr.nu = 1;                                 // either just 1, or combined into 1
 
 	if ( hdr.nt == 1 )
 	{
 		hdr.ndim = 3;
 	}
-	else if ( hdr.nu == 1 )
-	{
-		hdr.ndim = 4;
-	}
 	else
 	{
-		hdr.ndim = 5;
+		hdr.ndim = 4;
 	}
 	
 	hdr.nv = 1;
 	hdr.nw = 1;
 
-	hdr.nvox = hdr.nx * hdr.ny * hdr.nz * hdr.nt * hdr.nu;
+	hdr.nvox = (size_t)(hdr.nx * hdr.ny * hdr.nz * hdr.nt);
 
 	hdr.datatype = DT_SIGNED_SHORT;
 
@@ -209,7 +202,7 @@ bool NIfTICreateHeader(nifti_image &hdr, IMAGEMAP &imageMap)
 
 	if (! SliceDistance(hdr, imageList))
     {
-		cerr << endl << "\t^G^G**** CreateNIfTIHeader: Cannot find slice distance."<< endl << endl;
+		cerr << endl << "\t**** CreateNIfTIHeader: Cannot find slice distance."<< endl << endl;
 		return false;
     }
 
@@ -222,7 +215,7 @@ bool NIfTICreateHeader(nifti_image &hdr, IMAGEMAP &imageMap)
     }
 	else
     {
-		cerr << endl << "\t^G^G**** CreateNIfTIHeader: WARNING: number of repetitions is less than two" << endl
+		cerr << endl << "\t**** CreateNIfTIHeader: WARNING: number of repetitions is less than two" << endl
 			 << "\t**** time between volumes is set to zero" << endl;
 		hdr.dt = 0.0;
     }
@@ -264,13 +257,15 @@ bool NIfTICreateHeader(nifti_image &hdr, IMAGEMAP &imageMap)
 	hdr.intent_p2 = 0.0;
 	hdr.intent_p3 = 0.0;
 	hdr.intent_name[0] = '\0' ;
-	hdr.descrip[0] = '\0';
 	hdr.aux_file[0] = '\0';
 
 	hdr.byteorder = nifti_short_order();  /* use host byte order */
 
 	hdr.data = NULL; 
 
+	strncpy(hdr.descrip, image->ACQDescription().c_str(), 79);
+	hdr.descrip[79] = '\0';
+
 	return retValue;
 }
 
